package de.mattv.quarry

import net.minecraft.data.DataGenerator
import net.minecraft.data.recipes.FinishedRecipe
import net.minecraft.data.recipes.RecipeProvider
import net.minecraft.data.recipes.ShapedRecipeBuilder
import net.minecraft.data.tags.BlockTagsProvider
import net.minecraft.tags.BlockTags
import net.minecraft.world.item.Items
import net.minecraftforge.client.model.generators.BlockStateProvider
import net.minecraftforge.client.model.generators.ItemModelProvider
import net.minecraftforge.common.Tags
import net.minecraftforge.common.data.ExistingFileHelper
import net.minecraftforge.common.data.LanguageProvider
import net.minecraftforge.eventbus.api.SubscribeEvent
import net.minecraftforge.fml.common.Mod
import net.minecraftforge.forge.event.lifecycle.GatherDataEvent
import java.util.function.Consumer

@Mod.EventBusSubscriber(modid=QuarryMod.ID, bus=Mod.EventBusSubscriber.Bus.MOD)
object DataGenSubscriber {
    @SubscribeEvent
    fun gatherData(event: GatherDataEvent) {
        val gen = event.generator
        if (event.includeServer()) {
            gen.addProvider(DataGenBlockTags(gen, event.existingFileHelper))
            gen.addProvider(DataGenRecipes(gen))
        }
        if (event.includeClient()) {
            gen.addProvider(DataGenBlockStates(gen, event.existingFileHelper))
            gen.addProvider(DataGenItemModels(gen, event.existingFileHelper))
            gen.addProvider(DataGenTranslation(gen, "en_us"))
        }
    }
}

class DataGenBlockStates(gen: DataGenerator?, helper: ExistingFileHelper?) : BlockStateProvider(gen, QuarryMod.ID, helper) {
    override fun registerStatesAndModels() {
        val quarryBlock = QuarryMod.QUARRY_BLOCK.get()
        val quarryUpDown = modLoc("blocks/quarry/updown")
        val quarrySide = modLoc("blocks/quarry/side")
        simpleBlock(quarryBlock, models().cubeColumn(quarryBlock.registryName!!.path, quarrySide, quarryUpDown))
    }
}

class DataGenItemModels(gen: DataGenerator?, helper: ExistingFileHelper?) : ItemModelProvider(gen, QuarryMod.ID, helper) {
    override fun registerModels() {
        withExistingParent(QuarryMod.QUARRY_ITEM.get().registryName!!.path, modLoc("block/quarry"))
        singleTexture(QuarryMod.CONTROLLER_ITEM.get().registryName!!.path, mcLoc("item/generated"), "layer0", modLoc("items/controller"))
        singleTexture(QuarryMod.QUARRY_SPEED_UPGRADE.get().registryName!!.path, mcLoc("item/generated"), "layer0", modLoc("items/upgrade_speed"))
        singleTexture(QuarryMod.QUARRY_FORTUNE_UPGRADE.get().registryName!!.path, mcLoc("item/generated"), "layer0", modLoc("items/upgrade_fortune"))
    }
}

class DataGenBlockTags(gen: DataGenerator, helper: ExistingFileHelper?) : BlockTagsProvider(gen, QuarryMod.ID, helper) {
    override fun addTags() {
        tag(BlockTags.MINEABLE_WITH_PICKAXE)
            .add(QuarryMod.QUARRY_BLOCK.get())
        tag(BlockTags.NEEDS_IRON_TOOL)
            .add(QuarryMod.QUARRY_BLOCK.get())
    }

    override fun getName(): String = "MQuarry Tags"
}

class DataGenTranslation(gen: DataGenerator, locale: String) : LanguageProvider(gen, QuarryMod.ID, locale) {
    override fun addTranslations() {
        add(QuarryMod.QUARRY_BLOCK.get(), "MQuarry")
        add(QuarryMod.QUARRY_SPEED_UPGRADE.get(), "MQuarry speed upgrade")
        add(QuarryMod.QUARRY_FORTUNE_UPGRADE.get(), "MQuarry fortune upgrade")
        add(QuarryMod.CONTROLLER_ITEM.get(), "MQuarry controller")
    }
}

class DataGenRecipes(gen: DataGenerator) : RecipeProvider(gen) {
    override fun buildCraftingRecipes(consumer: Consumer<FinishedRecipe>) {
        ShapedRecipeBuilder.shaped(QuarryMod.QUARRY_ITEM.get())
            .pattern("eEe")
            .pattern("pRs")
            .pattern("DGI")
            .define('e', Tags.Items.ENDER_PEARLS)
            .define('p', Items.NETHERITE_PICKAXE)
            .define('s', Items.NETHERITE_SHOVEL)
            .define('E', Items.EMERALD_BLOCK)
            .define('R', Items.REDSTONE_BLOCK)
            .define('D', Items.DIAMOND_BLOCK)
            .define('G', Items.GOLD_BLOCK)
            .define('I', Items.IRON_BLOCK)
            .unlockedBy("never", net.minecraft.advancements.critereon.ImpossibleTrigger.TriggerInstance())
            .save(consumer)

        ShapedRecipeBuilder.shaped(QuarryMod.QUARRY_SPEED_UPGRADE.get())
            .pattern("sss")
            .pattern("dRd")
            .pattern("RIR")
            .define('s', Items.SUGAR)
            .define('d', Tags.Items.GEMS_DIAMOND)
            .define('R', Items.REDSTONE_BLOCK)
            .define('I', Items.IRON_BLOCK)
            .unlockedBy("never", net.minecraft.advancements.critereon.ImpossibleTrigger.TriggerInstance())
            .save(consumer)

        ShapedRecipeBuilder.shaped(QuarryMod.QUARRY_FORTUNE_UPGRADE.get())
            .pattern("LLL")
            .pattern("eRe")
            .pattern("RIR")
            .define('e', Tags.Items.GEMS_EMERALD)
            .define('R', Items.REDSTONE_BLOCK)
            .define('I', Items.IRON_BLOCK)
            .define('L', Items.LAPIS_BLOCK)
            .unlockedBy("never", net.minecraft.advancements.critereon.ImpossibleTrigger.TriggerInstance())
            .save(consumer)

        ShapedRecipeBuilder.shaped(QuarryMod.CONTROLLER_ITEM.get())
            .pattern("i i")
            .pattern("rrr")
            .pattern(" d ")
            .define('i', Tags.Items.INGOTS_IRON)
            .define('r', Tags.Items.DUSTS_REDSTONE)
            .define('d', Tags.Items.GEMS_DIAMOND)
            .unlockedBy("never", net.minecraft.advancements.critereon.ImpossibleTrigger.TriggerInstance())
            .save(consumer)
    }
}
